from __future__ import annotations

from flask import Blueprint
import ckan.plugins.toolkit as tk
import ckan.authz as authz
from ckan.lib.helpers import Page

CONFIG_BASE_TEMPLATE = "ckanext.check_link.report.base_template"
CONFIG_REPORT_URL = "ckanext.check_link.report.url"


DEFAULT_BASE_TEMPLATE = "check_link/base_admin.html"
DEFAULT_REPORT_URL = "/check-link/report/global"


bp = Blueprint("check_link", __name__)

def get_blueprints():
    report_url = tk.config.get(CONFIG_REPORT_URL, DEFAULT_REPORT_URL)
    if report_url:
        bp.add_url_rule(report_url, view_func=report)

    return [bp]


def report():
    if not authz.is_authorized_boolean("check_link_view_report_page", {"user": tk.g.user}, {}):
        return tk.abort(403)

    try:
        page = max(1, tk.asint(tk.request.args.get("page", 1)))
    except ValueError:
        page = 1

    per_page = 10
    reports = tk.get_action("check_link_report_search")({}, {
        "limit": per_page,
        "offset": per_page * page - per_page,
        "attached_only": True,
        "exclude_state": ["available"]
    })

    base_template = tk.config.get(
        CONFIG_BASE_TEMPLATE,
        DEFAULT_BASE_TEMPLATE
    )
    return tk.render("check_link/report.html", {
        "base_template": base_template,
        "page": Page(reports["results"], page=page, item_count=reports["count"], items_per_page=per_page, presliced_list=True)

    })



from __future__ import annotations

import click
import ckan.model as model
import ckan.plugins.toolkit as tk


def get_commands():
    return [check_link]


@click.group(short_help="Check link availability")
def check_link():
    pass


@check_link.command()
@click.option("-d", "--include-draft", is_flag=True, help="Check draft packages as well")
@click.option("-p", "--include-private", is_flag=True, help="Check private packages as well")
@click.argument("ids", nargs=-1)
def check_packages(include_draft: bool, include_private: bool, ids: tuple[str, ...]):
    """Check every resource inside each package.

    Scope can be narrowed via arbitary number of arguments, specifying
    package's ID or name.

    """
    user = tk.get_action("get_site_user")({"ignore_auth": True}, {})
    context = {"user": user["name"]}

    check = tk.get_action("check_link_package_check")
    states = ["active"]

    if include_draft:
        states.append("draft")

    q = model.Session.query(model.Package.id).filter(model.Package.state.in_(states))

    if not include_private:
        q = q.filter(model.Package.private == False)

    if ids:
        q = q.filter(model.Package.id.in_(ids) | model.Package.name.in_(ids))

    with click.progressbar(q, length=q.count()) as bar:
        for pkg in bar:
            check(context.copy(), {"id": pkg.id, "save": True, "clear_available": True})

    click.secho("Done", fg="green")



import pytest
import factory
from pytest_factoryboy import register
from ckan.tests.factories import CKANFactory, fake, Resource

from ckanext.check_link.model import Report



@pytest.fixture
def clean_db(reset_db, migrate_db_for):
    reset_db()
    migrate_db_for("check_link")


@register
class ReportFactory(CKANFactory):
    """A factory class for creating CKAN users."""

    class Meta:
        model = Report
        action = "check_link_report_save"

    # These are the default params that will be used to create new users.
    url = factory.LazyFunction(fake.url)
    resource_id = factory.LazyFunction(lambda: Resource()["id"])
    state = "available"
